package com.spring.dcaa.controller;

import com.spring.dcaa.dto.ReportDto;
import com.spring.dcaa.entity.Visitor;
import com.spring.dcaa.repository.VisitorRepository;
import jakarta.persistence.EntityNotFoundException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

@RestController
@RequiredArgsConstructor
@RequestMapping("/visitors")
@Slf4j
public class VisitorController {
    private final VisitorRepository visitorRepository;
    private final RestTemplate restTemplate;
    public static final String CORRELATION_ID = "correlation-id";

    @GetMapping("/report")
    public ResponseEntity<Object> externalReport(@RequestHeader(value = CORRELATION_ID) String correlationId) {
        String info = String.format("Fetching report from external service | correlation-id: %s | endpoint: /visitors/report | method: GET", correlationId);
        log.info(info);
        HttpHeaders headers = setHeaders(correlationId);
        String externalServiceUrl = "http://report:8082/visitors";
        ResponseEntity<Object> response = restTemplate.exchange(
                externalServiceUrl,
                HttpMethod.GET,
                new HttpEntity<>(headers),
                new ParameterizedTypeReference<>() {}
        );
        return new ResponseEntity<>(response.getBody(), headers, HttpStatus.OK);
    }

    @GetMapping
    public ResponseEntity<ReportDto> report(@RequestHeader(CORRELATION_ID) String correlationId) {
        String info = String.format("Report generated | correlation-id: %s | endpoint: /visitors | method: GET", correlationId);
        log.info(info);
        return new ResponseEntity<>(visitorRepository.generateReport(), setHeaders(correlationId), HttpStatus.OK);
    }

    @PostMapping
    public ResponseEntity<Long> add(@RequestBody Visitor visitor, @RequestHeader(CORRELATION_ID) String correlationId) {
        String info = String.format("Adding visitor | correlation-id: %s | endpoint: /visitors | method: POST", correlationId);
        log.info(info);
        return new ResponseEntity<>(visitorRepository.save(visitor).getId(), setHeaders(correlationId), HttpStatus.OK);
    }

    @PutMapping("/{id}")
    public ResponseEntity<Long> edit(@RequestBody Visitor visitor, @PathVariable Long id, @RequestHeader(CORRELATION_ID) String correlationId) {
        String info = String.format("Editing visitor | correlation-id: %s | endpoint: /visitors/%s | method: PUT", correlationId, id);
        log.info(info);
        Visitor visitorDb = visitorRepository.findById(id)
                .orElseThrow(() -> new EntityNotFoundException("Visitor with id " + id + " not found"));
        visitorDb.setName(visitor.getName());
        visitorDb.setSurname(visitor.getSurname());
        visitorDb.setPatronymic(visitor.getPatronymic());
        visitorDb.setHasDisk(visitor.getHasDisk());
        return new ResponseEntity<>(visitorRepository.save(visitorDb).getId(), setHeaders(correlationId), HttpStatus.OK);
    }

    @DeleteMapping("/{id}")
    public ResponseEntity<Void> delete(@PathVariable Long id, @RequestHeader(CORRELATION_ID) String correlationId) {
        String info = String.format("Deleting visitor | correlation-id: %s | endpoint: /visitors/%s | method: DELETE", correlationId, id);
        log.info(info);
        Visitor visitorDb = visitorRepository.findById(id)
                .orElseThrow(() -> new EntityNotFoundException("Visitor with id " + id + " not found"));
        visitorRepository.delete(visitorDb);
        return new ResponseEntity<>(setHeaders(correlationId), HttpStatus.OK);
    }

    private HttpHeaders setHeaders(String info) {
        HttpHeaders headers = new HttpHeaders();
        headers.set(CORRELATION_ID, info);
        return headers;
    }
}
