package com.spring.report.controller;

import com.opencsv.CSVWriter;
import com.spring.report.entity.Visitor;
import com.spring.report.kafka.Producer;
import com.spring.report.repository.VisitorRepository;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("/visitors")
@Slf4j
public class VisitorController {
    private final VisitorRepository visitorRepository;
    public static final String CORRELATION_ID = "correlation-id";
    private final Producer producer;

    @GetMapping
    public ResponseEntity<List<Visitor>> report(@RequestHeader(CORRELATION_ID) String correlationId) {
        log.info(String.format("Report generated | correlation-id: %s | endpoint: /visitors/report | method: GET", correlationId));

        List<Visitor> visitors = visitorRepository.findAll();

        String csvFileName = "visitors_report.csv";
        File csvFile = generateCsv(visitors, csvFileName);

        uploadFileToMinio(csvFile, correlationId);

        if (csvFile.exists()) {
            csvFile.delete();
        }

        return new ResponseEntity<>(visitorRepository.findAll(), setHeaders(correlationId), HttpStatus.OK);
    }

    private HttpHeaders setHeaders(String info) {
        HttpHeaders headers = new HttpHeaders();
        headers.set(CORRELATION_ID, info);
        return headers;
    }

    private File generateCsv(List<Visitor> visitors, String fileName) {
        File csvFile = new File(fileName);
        try (CSVWriter writer = new CSVWriter(new FileWriter(csvFile),
                ';',
                CSVWriter.NO_QUOTE_CHARACTER,
                CSVWriter.DEFAULT_ESCAPE_CHARACTER,
                CSVWriter.DEFAULT_LINE_END)) {
            String[] header = {"id", "name", "surname", "patronymic", "hasDisk"};
            writer.writeNext(header);

            for (Visitor visitor : visitors) {
                String[] data = {
                        String.valueOf(visitor.getId()),
                        visitor.getName(),
                        visitor.getSurname(),
                        visitor.getPatronymic(),
                        String.valueOf(visitor.getHasDisk())
                };
                writer.writeNext(data);
            }
        } catch (IOException e) {
            log.error("Error generating CSV", e);
        }
        return csvFile;
    }

    private void uploadFileToMinio(File file, String corId) {
        try {
            producer.sendEvent(Producer.TEST_TOPIC, corId, Files.readAllBytes(file.toPath()));
        } catch (Exception e) {
            log.error("Error uploading file to Kafka", e);
        }
    }
}
